X-Git-Url: https://www.chiark.greenend.org.uk/ucgi/~mdw/git/disorder/blobdiff_plain/460b9539a7c15580e41a71bbc0f47ae776238915..985bb670b4e07d35cb1580780253ded2524a342e:/lib/log.c
diff --git a/lib/log.c b/lib/log.c
index e6539cf..3f57073 100644
--- a/lib/log.c
+++ b/lib/log.c
@@ -1,31 +1,40 @@
/*
* This file is part of DisOrder.
- * Copyright (C) 2004, 2005, 2006 Richard Kettlewell
+ * Copyright (C) 2004-2008 Richard Kettlewell
*
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see .
+ */
+/** @file lib/log.c @brief Errors and logging
*
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
+ * All messages are initially emitted by one of the four functions below.
+ * debug() is generally invoked via D() so that mostly you just do a test
+ * rather than a complete subroutine call.
*
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * Messages are dispatched via @ref log_default. This defaults to @ref
+ * log_stderr. daemonize() will turn off @ref log_stderr and use @ref
+ * log_syslog instead.
+ *
+ * fatal() will call exitfn() with a nonzero status. The default value is
+ * exit(), but it should be set to _exit() anywhere but the 'main line' of the
+ * program, to guarantee that exit() gets called at most once.
*/
#define NO_MEMORY_ALLOCATION
/* because the memory allocation functions report errors */
-#include
+#include "common.h"
-#include
-#include
-#include
#include
#include
#include
@@ -34,30 +43,54 @@
#include "disorder.h"
#include "printf.h"
+/** @brief Definition of a log output */
struct log_output {
+ /** @brief Function to call */
void (*fn)(int pri, const char *msg, void *user);
+ /** @brief User data */
void *user;
};
+/** @brief Function to call on a fatal error
+ *
+ * This is normally @c exit() but in the presence of @c fork() it
+ * sometimes gets set to @c _exit(). */
void (*exitfn)(int) attribute((noreturn)) = exit;
+
+/** @brief Debug flag */
int debugging;
+
+/** @brief Program name */
const char *progname;
+
+/** @brief Filename for debug messages */
const char *debug_filename;
+
+/** @brief Line number for debug messages */
int debug_lineno;
+
+/** @brief Pointer to chosen log output structure */
struct log_output *log_default = &log_stderr;
+/** @brief Filename to debug for */
static const char *debug_only;
/* we might be receiving things in any old encoding, or binary rubbish in no
* encoding at all, so escape anything we don't like the look of */
static void format(char buffer[], size_t bufsize, const char *fmt, va_list ap) {
char t[1024];
const char *p;
int ch;
size_t n = 0;
- if(byte_vsnprintf(t, sizeof t, fmt, ap) < 0)
- strcpy(t, "[byte_vsnprintf failed]");
+ if(byte_vsnprintf(t, sizeof t, fmt, ap) < 0) {
+ strcpy(t, "[byte_vsnprintf failed: ");
+ strncat(t, fmt, sizeof t - strlen(t) - 1);
+ }
p = t;
while((ch = (unsigned char)*p++)) {
if(ch >= ' ' && ch <= 126) {
@@ -74,7 +107,11 @@ static void format(char buffer[], size_t bufsize, const char *fmt, va_list ap) {
buffer[n] = 0;
}
-/* log to a file */
+/** @brief Log to a file
+ * @param pri Message priority (as per syslog)
+ * @param msg Messagge to log
+ * @param user The @c FILE @c * to log to or NULL for @c stderr
+ */
static void logfp(int pri, const char *msg, void *user) {
struct timeval tv;
FILE *fp = user ? user : stderr;
@@ -106,7 +143,7 @@ static void logfp(int pri, const char *msg, void *user) {
fputc('\n', fp);
}
-/* log to syslog */
+/** @brief Log to syslog */
static void logsyslog(int pri, const char *msg,
void attribute((unused)) *user) {
if(pri < LOG_DEBUG)
@@ -115,10 +152,13 @@ static void logsyslog(int pri, const char *msg,
syslog(pri, "%s:%d: %s", debug_filename, debug_lineno, msg);
}
+/** @brief Log output that writes to @c stderr */
struct log_output log_stderr = { logfp, 0 };
+
+/** @brief Log output that sends to syslog */
struct log_output log_syslog = { logsyslog, 0 };
-/* log to all log outputs */
+/** @brief Format and log a message */
static void vlogger(int pri, const char *fmt, va_list ap) {
char buffer[1024];
@@ -126,7 +166,7 @@ static void vlogger(int pri, const char *fmt, va_list ap) {
log_default->fn(pri, buffer, log_default->user);
}
-/* wrapper for vlogger */
+/** @brief Format and log a message */
static void logger(int pri, const char *fmt, ...) {
va_list ap;
@@ -135,7 +175,12 @@ static void logger(int pri, const char *fmt, ...) {
va_end(ap);
}
-/* internals of fatal/error/info */
+/** @brief Format and log a message
+ * @param pri Message priority (as per syslog)
+ * @param fmt Format string
+ * @param errno_value Errno value to include as a string, or 0
+ * @param ap Argument list
+ */
void elog(int pri, int errno_value, const char *fmt, va_list ap) {
char buffer[1024];
@@ -149,14 +194,41 @@ void elog(int pri, int errno_value, const char *fmt, va_list ap) {
}
}
-#define disorder_fatal fatal
-#define disorder_error error
-#define disorder_info info
+/** @brief Log an error and quit
+ *
+ * If @c ${DISORDER_FATAL_ABORT} is defined (as anything) then the process
+ * is aborted, so you can get a backtrace.
+ */
+void disorder_fatal(int errno_value, const char *msg, ...) {
+ va_list ap;
+
+ va_start(ap, msg);
+ elog(LOG_CRIT, errno_value, msg, ap);
+ va_end(ap);
+ if(getenv("DISORDER_FATAL_ABORT")) abort();
+ exitfn(EXIT_FAILURE);
+}
+
+/** @brief Log an error */
+void disorder_error(int errno_value, const char *msg, ...) {
+ va_list ap;
-/* shared implementation of vararg functions */
-#include "log-impl.h"
+ va_start(ap, msg);
+ elog(LOG_ERR, errno_value, msg, ap);
+ va_end(ap);
+}
+
+/** @brief Log an informational message */
+void disorder_info(const char *msg, ...) {
+ va_list ap;
+
+ va_start(ap, msg);
+ elog(LOG_INFO, 0, msg, ap);
+ va_end(ap);
+}
-void debug(const char *msg, ...) {
+/** @brief Log a debug message */
+void disorder_debug(const char *msg, ...) {
va_list ap;
va_start(ap, msg);
@@ -164,6 +236,7 @@ void debug(const char *msg, ...) {
va_end(ap);
}
+/** @brief Set the program name from @c argc */
void set_progname(char **argv) {
if((progname = strrchr(argv[0], '/')))
++progname;
@@ -177,4 +250,3 @@ c-basic-offset:2
comment-column:40
End:
*/
-/* arch-tag:78385d5240eab4439cb7eca7dad5154d */
@@ -34,30 +43,54 @@
static const char *debug_only;
-/* we might be receiving things in any old encoding, or binary rubbish in no
- * encoding at all, so escape anything we don't like the look of */
+/** @brief Construct log line, encoding special characters
+ *
+ * We might be receiving things in any old encoding, or binary rubbish
+ * in no encoding at all, so escape anything we don't like the look
+ * of. We limit the log message to a kilobyte.
+ */
static void format(char buffer[], size_t bufsize, const char *fmt, va_list ap) {
char t[1024];